@@ -1,27 +1,32 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2014 -- leonerd@leonerd.org.uk
 
 package Future::Utils;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.23';
 
 use Exporter 'import';
 
 our @EXPORT_OK = qw(
+   call
+   call_with_escape
+
    repeat
+   try_repeat try_repeat_until_success
    repeat_until_success
 
-   fmap fmap_concat
-   fmap1
-   fmap_void
+   fmap  fmap_concat
+   fmap1 fmap_scalar
+   fmap0 fmap_void
 );
 
 use Carp;
+our @CARP_NOT = qw( Future );
 
 =head1 NAME
 
@@ -29,7 +34,18 @@ C<Future::Utils> - utility functions for working with C<Future> objects
 
 =head1 SYNOPSIS
 
- use Future::Utils qw( repeat );
+ use Future::Utils qw( call_with_escape );
+
+ my $result_f = call_with_escape {
+    my $escape_f = shift;
+    my $f = ...
+       $escape_f->done( "immediate result" );
+       ...
+ };
+
+Z<>
+
+ use Future::Utils qw( repeat try_repeat try_repeat_until_success );
 
  my $eventual_f = repeat {
     my $trial_f = ...
@@ -47,12 +63,17 @@ C<Future::Utils> - utility functions for working with C<Future> objects
     return $trial_f;
  } foreach => \@items;
 
- my $eventual_f = repeat_until_success {
+ my $eventual_f = try_repeat {
+    my $trial_f = ...
+    return $trial_f;
+ } while => sub { ... };
+
+ my $eventual_f = try_repeat_until_success {
     ...
     return $trial_f;
  };
 
- my $eventual_f = repeat_until_success {
+ my $eventual_f = try_repeat_until_success {
     my $item = shift;
     ...
     return $trial_f;
@@ -60,13 +81,15 @@ C<Future::Utils> - utility functions for working with C<Future> objects
 
 Z<>
 
- my $result_f = fmap {
+ use Future::Utils qw( fmap_concat fmap_scalar fmap_void );
+
+ my $result_f = fmap_concat {
     my $item = shift;
     ...
     return $item_f;
  } foreach => \@items, concurrent => 4;
 
- my $result_f = fmap1 {
+ my $result_f = fmap_scalar {
     my $item = shift;
     ...
     return $item_f;
@@ -80,6 +103,59 @@ Z<>
 
 =cut
 
+=head1 INVOKING A BLOCK OF CODE
+
+=head2 $f = call { CODE }
+
+The C<call> function invokes a block of code that returns a future, and simply
+returns the future it returned. The code is wrapped in an C<eval {}> block, so
+that if it throws an exception this is turned into an immediate failed
+C<Future>. If the code does not return a C<Future>, then an immediate failed
+C<Future> instead.
+
+(This is equivalent to using C<< Future->call >>, but is duplicated here for
+completeness).
+
+=cut
+
+sub call(&)
+{
+   my ( $code ) = @_;
+   return Future->call( $code );
+}
+
+=head2 $f = call_with_escape { CODE }
+
+The C<call_with_escape> function invokes a block of code that returns a
+future, and passes in a separate future (called here an "escape future").
+Normally this is equivalent to the simple C<call> function. However, if the
+code captures this future and completes it by calling C<done> or C<fail> on
+it, the future returned by C<call_with_escape> immediately completes with this
+result, and the future returned by the code itself is cancelled.
+
+This can be used to implement short-circuit return from an iterating loop or
+complex sequence of code, or immediate fail that bypasses failure handling
+logic in the code itself, or several other code patterns.
+
+ $f = $code->( $escape_f )
+
+(This can be considered similar to C<call-with-escape-continuation> as found
+in some Scheme implementations).
+
+=cut
+
+sub call_with_escape(&)
+{
+   my ( $code ) = @_;
+
+   my $escape_f = Future->new;
+
+   return Future->wait_any(
+      Future->call( $code, $escape_f ),
+      $escape_f,
+   );
+}
+
 =head1 REPEATING A BLOCK OF CODE
 
 The C<repeat> function provides a way to repeatedly call a block of code that
@@ -95,11 +171,8 @@ If the eventual future is cancelled, the latest trial future will be
 cancelled.
 
 If some specific subclass or instance of C<Future> is required as the return
-value, it can be passed as the C<return> argument. Otherwise, for backward
-compatibility, the return C<Future> will be constructed by cloning the first
-trial C<Future>. Because this design has been found to be poor, this will be
-removed in a later version. If the trial C<Future> is of some other subclass,
-a warning will be printed about this impending change of behaviour.
+value, it can be passed as the C<return> argument. Otherwise the return value
+will be constructed by cloning the first non-immediate trial C<Future>.
 
 =head2 $future = repeat { CODE } while => CODE
 
@@ -176,20 +249,35 @@ it should return an empty list.
 
 sub _repeat
 {
-   my ( $code, $future, $running, $cond, $sense ) = @_;
-   $$running->on_ready( sub {
-      my $self = shift;
-      my $again = !!$cond->( $self ) ^ $sense;
-      if( $again ) {
-         $$running = Future->call( $code, $self );
-         _repeat( $code, $future, $running, $cond, $sense );
+   my ( $code, $return, $trialp, $cond, $sense ) = @_;
+
+   my $prev = $$trialp;
+
+   while(1) {
+      my $trial = $$trialp ||= Future->call( $code, $prev );
+      $prev = $trial;
+
+      if( !$trial->is_ready ) {
+         # defer
+         $return ||= $trial->new;
+         $trial->on_ready( sub {
+            return if $$trialp->is_cancelled;
+            _repeat( $code, $return, $trialp, $cond, $sense );
+         });
+         return $return;
       }
-      else {
-         # Propagate result
-         $$running->on_done( $future );
-         $$running->on_fail( $future );
+
+      if( !$cond->( $trial ) ^ $sense ) {
+         # Return result
+         $return ||= $trial->new;
+         $trial->on_done( $return );
+         $trial->on_fail( $return );
+         return $return;
       }
-   } );
+
+      # redo
+      undef $$trialp;
+   }
 }
 
 sub repeat(&@)
@@ -253,47 +341,56 @@ sub repeat(&@)
       }
    }
 
-   my $future;
-   my $running;
-   if( $args{return} ) {
-      $future = $args{return};
-      $running = Future->call( $code );
-   }
-   else {
-      $running = Future->call( $code );
-      $future = $running->new;
-      if( ref $future ne "Future" ) {
-         carp "Using a subclassed Trial Future for cloning is deprecated; use the 'return' argument instead"
-      }
-   }
+   my $future = $args{return};
 
-   $args{while} and _repeat( $code, $future, \$running, $args{while}, 0 );
-   $args{until} and _repeat( $code, $future, \$running, $args{until}, 1 );
+   my $trial;
+   $args{while} and $future = _repeat( $code, $future, \$trial, $args{while}, 0 );
+   $args{until} and $future = _repeat( $code, $future, \$trial, $args{until}, 1 );
 
-   $future->on_cancel( sub { $running->cancel } );
+   $future->on_cancel( sub { $trial->cancel } );
 
    return $future;
 }
 
-=head2 $future = repeat_until_success { CODE } ...
+=head2 $future = try_repeat { CODE } ...
 
-A shortcut to calling C<repeat> with an ending condition that simply tests for
-a successful result from a future. May be combined with C<foreach> or
-C<generate>.
+Currently a simple alias to C<repeat>. However, in some later version the
+C<repeat> function will be changed so that if a trial future fails, then the
+eventual future will immediately fail as well, making its semantics a little
+closer to that of a C<while {}> loop in Perl. Code that specifically wishes
+to catch failures in trial futures and retry the block should use
+C<try_repeat> specifically.
 
 =cut
 
-sub repeat_until_success(&@)
+*try_repeat = \&repeat;
+
+=head2 $future = try_repeat_until_success { CODE } ...
+
+A shortcut to calling C<try_repeat> with an ending condition that simply tests
+for a successful result from a future. May be combined with C<foreach> or
+C<generate>.
+
+This function used to be called C<repeat_until_success>, and is currently
+aliased as this name as well.
+
+=cut
+
+sub try_repeat_until_success(&@)
 {
    my $code = shift;
    my %args = @_;
 
+   # TODO: maybe merge while/until conditions one day...
    defined($args{while}) or defined($args{until})
-      and croak "Cannot pass 'while' or 'until' to repeat_until_success";
+      and croak "Cannot pass 'while' or 'until' to try_repeat_until_success";
 
-   repeat \&$code, while => sub { shift->failure }, %args;
+   try_repeat( $code, while => sub { shift->failure }, %args );
 }
 
+# Legacy name
+*repeat_until_success = \&try_repeat_until_success;
+
 =head1 APPLYING A FUNCTION TO A LIST
 
 The C<fmap> family of functions provide a way to call a block of code that
@@ -320,7 +417,9 @@ The following named arguments are common to each C<fmap*> function:
 
 Provides the list of items to iterate over, as an C<ARRAY> reference.
 
-The referenced array may be modified by this operation.
+The referenced array will be modified by this operation, C<shift>ing one item
+from it each time. The can C<push> more items to this array as it runs, and
+they will be included in the iteration.
 
 =item generate => CODE
 
@@ -330,6 +429,9 @@ empty list to indicate it has no more items.
 
  ( $item ) = $generate->()
 
+This function will be invoked each time any previous item future has completed
+and may be called again even after it has returned empty.
+
 =item concurrent => INT
 
 Gives the number of item futures to keep outstanding. By default this value
@@ -338,9 +440,10 @@ futures will be started at once.
 
 =item return => Future
 
-Normally, a new instance is returned of the base C<Future> class. By passing
-a new instance as the C<return> keyword, the result will be put into the given
-instance. This can be used to return subclasses, or specific instances.
+Normally, a new instance is returned by cloning the first non-immediate future
+returned as an item future. By passing a new instance as the C<return>
+argument, the result will be put into the given instance. This can be used to
+return subclasses, or specific instances.
 
 =back
 
@@ -355,34 +458,77 @@ below.
 
 =cut
 
+# This function is invoked in two circumstances:
+#  a) to create an item Future in a slot,
+#  b) once a non-immediate item Future is complete, to check its results
+# It can tell which circumstance by whether the slot itself is defined or not
 sub _fmap_slot
 {
-   my @args = my ( $slots, $idx, $code, $generator, $collect, $results, $future ) = @_;
+   my ( $slots, undef, $code, $generator, $collect, $results, $return ) = @_;
 
-   if( my ( $item ) = $generator->() ) {
-      my $f = $slots->[$idx] = Future->call( $code, $item );
+   SLOT: while(1) {
+      # Capture args each call because we mutate them
+      my ( undef, $idx ) = my @args = @_;
 
-      if( $collect eq "array" ) {
-         push @$results, my $r = [];
-         $f->on_done( sub { @$r = @_; _fmap_slot( @args ); });
+      unless( $slots->[$idx] ) {
+         # No item Future yet (case a), so create one
+         my $item;
+         unless( ( $item ) = $generator->() ) {
+            # All out of items, so now just wait for the slots to be finished
+            undef $slots->[$idx];
+            defined and return $return for @$slots;
+
+            # All the slots are done
+            $return ||= Future->new;
+
+            $return->done( @$results );
+            return $return;
+         }
+
+         my $f = $slots->[$idx] = Future->call( $code, $item );
+
+         if( $collect eq "array" ) {
+            push @$results, my $r = [];
+            $f->on_done( sub { @$r = @_ });
+         }
+         elsif( $collect eq "scalar" ) {
+            push @$results, undef;
+            my $r = \$results->[-1];
+            $f->on_done( sub { $$r = $_[0] });
+         }
       }
-      elsif( $collect eq "scalar" ) {
-         push @$results, undef;
-         my $r = \$results->[-1];
-         $f->on_done( sub { $$r = $_[0]; _fmap_slot( @args ); });
+
+      my $f = $slots->[$idx];
+
+      # Slot is non-immediate; arrange for us to be invoked again later when it's ready
+      if( !$f->is_ready ) {
+         $args[-1] = ( $return ||= $f->new );
+         $f->on_done( sub { _fmap_slot( @args ) } );
+         $f->on_fail( $return );
+
+         # Try looking for more that might be ready
+         my $i = $idx + 1;
+         while( $i != $idx ) {
+            $i++;
+            $i %= @$slots;
+            next if defined $slots->[$i];
+
+            $_[1] = $i;
+            redo SLOT;
+         }
+         return $return;
       }
-      else {
-         $f->on_done( sub { _fmap_slot( @args ); });
+
+      # Either we've been invoked again (case b), or the immediate Future was
+      # already ready.
+      if( $f->failure ) {
+         $return ||= $f->new;
+         $return->fail( $f->failure );
+         return $return;
       }
 
-      $f->on_fail( $future );
-      $future->on_cancel( $f );
-   }
-   else {
       undef $slots->[$idx];
-      defined and return for @$slots;
-
-      $future->done( @$results );
+      # next
    }
 }
 
@@ -392,10 +538,10 @@ sub _fmap
    my %args = @_;
 
    my $concurrent = $args{concurrent} || 1;
-   my @slots = ( undef ) x $concurrent;
+   my @slots;
 
    my $results = [];
-   my $future = $args{return} || Future->new;
+   my $future = $args{return};
 
    my $generator;
    if( $generator = $args{generate} ) {
@@ -409,16 +555,22 @@ sub _fmap
    }
 
    # If any of these immediately fail, don't bother continuing
-   $future->is_ready || _fmap_slot( \@slots, $_, $code, $generator, $args{collect}, $results, $future ) for 0 .. $#slots;
+   foreach my $idx ( 0 .. $concurrent-1 ) {
+      $future = _fmap_slot( \@slots, $idx, $code, $generator, $args{collect}, $results, $future );
+      last if $future->is_ready;
+   }
 
    $future->on_fail( sub {
       !defined $_ or $_->is_ready or $_->cancel for @slots;
    });
+   $future->on_cancel( sub {
+      $_->cancel for @slots;
+   });
 
    return $future;
 }
 
-=head2 $future = fmap { CODE } ...
+=head2 $future = fmap_concat { CODE } ...
 
 This version of C<fmap> expects each item future to return a list of zero or
 more values, and the overall result will be the concatenation of all these
@@ -427,17 +579,15 @@ results. It acts like a future-based equivalent to Perl's C<map> operator.
 The results are returned in the order of the original input values, not in the
 order their futures complete in. Because of the intermediate storage of
 C<ARRAY> references and final flattening operation used to implement this
-behaviour, this function is slightly less efficient than C<fmap1> or
+behaviour, this function is slightly less efficient than C<fmap_scalar> or
 C<fmap_void> in cases where item futures are expected only ever to return one,
 or zero values, respectively.
 
-This function is also available under the name C<fmap_concat> to emphasise the
-concatenation behaviour.
+This function is also available under the name of simply C<fmap> to emphasise
+its similarity to perl's C<map> keyword.
 
 =cut
 
-*fmap = \&fmap_concat; # alias
-
 sub fmap_concat(&@)
 {
    my $code = shift;
@@ -447,8 +597,9 @@ sub fmap_concat(&@)
       return Future->new->done( map { @$_ } @_ );
    });
 }
+*fmap = \&fmap_concat;
 
-=head2 $future = fmap1 { CODE } ...
+=head2 $future = fmap_scalar { CODE } ...
 
 This version of C<fmap> acts more like the C<map> functions found in Scheme or
 Haskell; it expects that each item future returns only one value, and the
@@ -457,15 +608,18 @@ items. If an item future returns more than one value the others will be
 discarded. If it returns no value, then C<undef> will be substituted in its
 place so that the result list remains in correspondence with the input list.
 
+This function is also available under the shorter name of C<fmap1>.
+
 =cut
 
-sub fmap1(&@)
+sub fmap_scalar(&@)
 {
    my $code = shift;
    my %args = @_;
 
    _fmap( $code, %args, collect => "scalar" )
 }
+*fmap1 = \&fmap_scalar;
 
 =head2 $future = fmap_void { CODE } ...
 
@@ -478,6 +632,8 @@ to control concurrency of a function call iterating over a list of items,
 obtaining its results by some other means (such as side-effects on captured
 variables, or some external system).
 
+This function is also available under the shorter name of C<fmap0>.
+
 =cut
 
 sub fmap_void(&@)
@@ -487,6 +643,7 @@ sub fmap_void(&@)
 
    _fmap( $code, %args, collect => "void" )
 }
+*fmap0 = \&fmap_void;
 
 =head1 AUTHOR
 
